package com.gyf.csams.association.model

import android.app.Application
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.google.gson.reflect.TypeToken
import com.gyf.csams.module.*
import com.gyf.lib.model.ScrollViewModel
import com.gyf.lib.uikit.AsyncStringForm
import com.gyf.lib.uikit.StringForm
import com.gyf.lib.util.*
import com.orhanobut.logger.Logger
import kotlinx.coroutines.launch

/**
 * 题库界面类型
 *
 */
enum class ExamActivityType(val menuName: String) {
    //入团题库
    SET_EXAM("入团题库"),

    //入团申请表
    JOIN_Association("入团申请表"),

    //查看答卷
    Answer("查看答卷")
}



/**
 * 问题长度
 */
const val QUESTION_TEXT_LENGTH = 30

/**
 * 选择题选项数
 */
const val ANSWER_SIZE = 4

/**
 * 答案长度
 *
 */
const val ANSWER_TEXT_LENGTH = 15


/**
 * 题型
 *
 */
enum class ExamType(val type: String) {
    //选择题
    CQ("选择题"),

    //开放题
    OQ("开放题")
}

abstract class Exam() {
    abstract val examType: ExamType
    abstract val _question: StringForm

    abstract fun check(): Boolean
    abstract val questionId: Int?
}

/**
 * 选择题
 *
 *
 */
data class ChoiceQuestionVo(
    override val examType: ExamType = ExamType.CQ,
    val _answers: MutableList<StringForm>,
    val rightAnswer: Int,
    val chooseOption: Int? = null,
    override val _question: StringForm,
    override val questionId: Int? = null,

    ) : Exam() {

    override fun check(): Boolean {
        if (_question.formValue.value?.isEmpty() == true || _question.formValue.value == null) {
            return false
        }
        _answers.forEach {
            if (it.formValue.value == null || it.formValue.value?.isEmpty() == true) {
                return false
            }
        }
        return true
    }
}


/**
 * 开放题
 *
 */
data class OpenQuestionsVo(
    override val examType: ExamType = ExamType.OQ,
    override val _question: StringForm,
    override val questionId: Int?,

    ) : Exam() {
    override fun check(): Boolean {
        TODO("Not yet implemented")
    }
}

/**
 * 题库状态管理
 *
 */

class ExamViewModel(application: Application) : ScrollViewModel<Exam>(application) {
    val questionIsNull: String = "问题或答案为空"
    val deleteTip = "确定删除此题目？"
    val addTip = "确定添加此题目？"
    val actionLabel = "确定"


    private val _newExam: MutableLiveData<Exam> = MutableLiveData()
    val newExam: LiveData<Exam> = _newExam

    private val _deleteIds = mutableListOf<Int>()

    private val _allowPostAnswer = MutableLiveData<Boolean>()
    val allowPostAnswer: LiveData<Boolean> = _allowPostAnswer

    /**
     * 创建题目
     *
     * @return
     */
    private fun createExam(examVo: ExamVo? = null, answer: Int? = null): Exam {
        val questionForm = AsyncStringForm(formDesc = "问题", textLength = QUESTION_TEXT_LENGTH)
        val answerA = AsyncStringForm(formDesc = "选项A", textLength = ANSWER_TEXT_LENGTH)
        val answerB = AsyncStringForm(formDesc = "选项B", textLength = ANSWER_TEXT_LENGTH)
        val answerC = AsyncStringForm(formDesc = "选项C", textLength = ANSWER_TEXT_LENGTH)
        val answerD = AsyncStringForm(formDesc = "选项D", textLength = ANSWER_TEXT_LENGTH)
        if (examVo != null) {
            questionForm.setValue(examVo.question)
            answerA.setValue(examVo.optionsA)
            answerB.setValue(examVo.optionsB)
            answerC.setValue(examVo.optionsC)
            answerD.setValue(examVo.optionsD)
            return ChoiceQuestionVo(
                _answers = mutableListOf(answerA, answerB, answerC, answerD),
                rightAnswer = examVo.rightOption,
                _question = questionForm,
                questionId = examVo.questionId,
                chooseOption = answer
            )
        } else {
            return ChoiceQuestionVo(
                _answers = mutableListOf(answerA, answerB, answerC, answerD),
                rightAnswer = 0,
                _question = questionForm,
            )
        }


    }

    fun checkChooseAnswer() {
        _data.value?.forEach {
            if (it is ChoiceQuestionVo && it.chooseOption == null) {
                _allowPostAnswer.postValue(false)
                return
            }
        }
        _allowPostAnswer.postValue(true)
    }

    /**
     * 更新题目
     *
     */
    fun update(oldExam: Exam, newExam: Exam) {
        if (oldExam == _newExam.value) {
            _newExam.value = newExam
        } else {
            _data.value?.apply {
                this[indexOf(oldExam)] = newExam
                val list = mutableListOf<Exam>()
                list.addAll(this)
                _data.value?.clear()
                _data.value = list
            }
        }
    }

    /**
     * 更新题库
     *
     * @param callback
     */
    fun updateExam(associationId: Int, callback: (message: String) -> Unit) {
        viewModelScope.launch {
            _data.value?.dropLast(1)?.mapNotNull { it ->
                if (it is ChoiceQuestionVo) {
                    toExamVo(choiceQuestionVo = it)
                } else {
                    null
                }
            }?.apply {
                HttpClient.post(url = Api.buildUrl(AssociationApi.UpdateExam),
                    callback = HttpCallback<Boolean>(action = "更新题库",
                        onSuccess = {
                            callback(it.message)
                        }, typeToken = object : TypeToken<ApiResponse<Boolean>>() {}.type
                    ),
                    jsonParam = AddExamVo(
                        questions = this,
                        associationId = associationId,
                        token = TokenManager.getToken(),
                        deleteIds = _deleteIds
                    )
                )
            }
        }
    }

    private fun toExamVo(choiceQuestionVo: ChoiceQuestionVo): ExamVo? {
        return choiceQuestionVo.let {
            val options = it._answers.map {
                it.getValue()
            }
            if (options.size == 4) {
                ExamVo(
                    question = it._question.getValue(), optionsA = options[0],
                    optionsB = options[1], optionsC = options[2], optionsD = options[3],
                    rightOption = it.rightAnswer,

                    questionId = it.questionId
                )
            } else {
                null
            }
        }
    }

    /**
     * 提交答卷
     * @param callback
     */
    fun postAnswer(associationId: Int, callback: (message: String) -> Unit) {
        viewModelScope.launch {
            _data.value?.mapNotNull { it ->
                if (it is ChoiceQuestionVo) {
                    toExamVo(choiceQuestionVo = it)?.let { choice ->
                        ApplyAnswerVo(
                            examVo = choice,
                            answer = it.chooseOption
                                ?: throw IllegalArgumentException("题目[id=${choice.questionId}]没有填写答案？？？")
                        )
                    }.apply {
                        Logger.i("$this")
                    }
                } else {
                    null
                }
            }?.let {
                Logger.i("提交${it.size}道题目答案")
                HttpClient.post(url = Api.buildUrl(AssociationApi.ApplyAnswer),
                    callback = HttpCallback<Boolean>(action = "提交答案",
                        onSuccess = {
                            callback(it.message)
                        }, typeToken = object : TypeToken<ApiResponse<Boolean>>() {}.type
                    ),
                    jsonParam = AddAnswerVo(
                        answers = it, token = TokenManager.getToken(),
                        associationId = associationId
                    )
                )
            }

        }
    }

    /**
     * 加载题库
     *
     */
    fun load(associationId: Int, activityType: ExamActivityType) {
        viewModelScope.launch {
            HttpClient.post(
                url = Api.buildUrl(
                    when (activityType) {
                        ExamActivityType.SET_EXAM -> AssociationApi.LoadExam
                        ExamActivityType.JOIN_Association -> AssociationApi.CreatePaper
                        else -> throw IllegalArgumentException("非法类型：${activityType}")
                    }
                ),
                callback = HttpCallback<MutableList<ExamVo>>(action = "加载题库", onSuccess = { it ->
                    it.body?.let { it1 ->
                        _data.postValue(it1.map {
                            createExam(examVo = it)
                        }.toMutableList().apply {
                            if (activityType == ExamActivityType.SET_EXAM) {
                                add(createExam())
                            }
                        })
                    }
                }, typeToken = object : TypeToken<ApiResponse<MutableList<ExamVo>>>() {}.type),
                jsonParam =
                SearchExamVo(associationId = associationId, token = TokenManager.getToken())
            )
        }
    }

    /**
     * 加载答卷
     *
     * @param joinId
     * @param callback
     */
    fun showAnswer(joinId: Int, callback: (message: String) -> Unit) {
        viewModelScope.launch {
            HttpClient.post(
                url = Api.buildUrl(AssociationApi.ShowAnswers),
                callback = HttpCallback<List<ApplyAnswerVo>>(action = "查看答案", onSuccess = { it ->
                    it.body?.apply {
                        val exam = map {
                            createExam(examVo = it.examVo, answer = it.answer)
                        }
                        _data.postValue(exam.toMutableList())
                    }
                    callback(it.message)
                }, typeToken = object : TypeToken<ApiResponse<List<ApplyAnswerVo>>>() {}.type),
                jsonParam = ShowAnswerVo(joinId = joinId, token = TokenManager.getToken())
            )
        }
    }


    /**
     * 添加题目
     *
     */
    fun addQuestion() {
        _data.value?.apply {
            _newExam.value?.let {
                val list = mutableListOf<Exam>()
                list.addAll(this)
                list.add(it)
                _data.postValue(list)
            }
            _newExam.value = createExam()
        }
    }

    /**
     *删除题目
     */
    fun deleteQuestion(exam: Exam) {
        _data.value?.apply {
            val list = mutableListOf<Exam>()
            remove(exam)
            list.addAll(this)
            Logger.i("size=${list.size}")
            _data.value?.clear()
            _data.postValue(list)
            exam.questionId?.let {
                _deleteIds.add(it)
            }
        }
    }
}