package com.gyf.csams.association.model

import android.app.Application
import android.graphics.Bitmap
import android.net.Uri
import androidx.core.graphics.drawable.toBitmap
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import coil.imageLoader
import coil.request.ImageRequest
import com.google.gson.reflect.TypeToken
import com.gyf.csams.MainApplication
import com.gyf.csams.module.ApiResponse
import com.gyf.csams.module.AssociationCheckVo
import com.gyf.csams.module.AssociationRegVo
import com.gyf.csams.module.ClientType
import com.gyf.lib.BuildConfig
import com.gyf.lib.model.ChoosePhotoViewModel
import com.gyf.lib.uikit.AsyncStringForm
import com.gyf.lib.uikit.FormStatus
import com.gyf.lib.util.*
import com.orhanobut.logger.Logger
import kotlinx.coroutines.launch
import java.io.File
import java.io.FileOutputStream


class RegAssociationViewModel(application: Application) : ChoosePhotoViewModel(application) {

    val frameDesc = "社团注册资料"

    val name = AsyncStringForm(formDesc = "社团名称", textLength = 5)
    val desc = AsyncStringForm(formDesc = "社团简介", textLength = 30)

    private val _fileId = MutableLiveData<Int>()
    val fileId: LiveData<Int> = _fileId

    private val _checkInfo = MutableLiveData<ApiResponse<AssociationCheckVo>>()
    val checkInfo: LiveData<ApiResponse<AssociationCheckVo>> = _checkInfo

    val picturePlaceHolder = "请上传图片"

    val errorPicture = "图片加载失败，请联系管理员"

    init {
        read()
    }


    /**
     * 加载历史提交的注册资料，可能为空
     *
     */
    private fun read() {
        viewModelScope.launch {
            HttpClient.post(
                Api.buildUrl(AssociationApi.Read),
                HttpCallback<AssociationCheckVo>(action = "加载历史注册资料", onSuccess = { it ->
                    _checkInfo.postValue(it)
                    it.body?.let { checkVo ->
                        name.setValue(checkVo.associationVo.name)
                        desc.setValue(checkVo.associationVo.desc)

                        val context = getApplication<Application>()
                        val request = ImageRequest.Builder(context)
                            .data("${BuildConfig.SERVER_ADDRESS}/${checkVo.associationVo.logo}")
                            .target(
                                onSuccess = { result ->

                                    checkVo.associationVo.logo.split("/").apply {
                                        File.createTempFile(last(), null, context.cacheDir).apply {
                                            Logger.d("文件路径：${absolutePath}")
                                            FileOutputStream(this).use {
                                                result.toBitmap().compress(
                                                    Bitmap.CompressFormat.JPEG,
                                                    100,
                                                    it
                                                )
                                            }
                                        }
                                            .apply {
                                                Logger.i("读取缓存图片")
                                                setPicture(Uri.fromFile(this))
                                                _fileId.postValue(checkVo.fileId)
                                            }

                                    }
                                }
                            )
                            .build()
                        context.imageLoader.enqueue(request)
                    }
                },
                    typeToken = object : TypeToken<ApiResponse<AssociationCheckVo>>() {}.type
                ),
                jsonParam = OnlyToken(clientType = ClientType.Foreground)
            )
        }
    }


    fun uploadPhoto(callback: (value: String) -> Unit) {
        getInputSteam()?.readBytes()?.apply {
            viewModelScope.launch {
                val context = getApplication<MainApplication>()

                runCatching {
                    val cacheFile = File(context.cacheDir, "${System.currentTimeMillis()}")
                    cacheFile.writeBytes(this@apply)

                    HttpClient.uploadFile(
                        Api.buildUrl(AssociationApi.Logo),
                        HttpCallback<List<Int>>("上传图片", onSuccess = {
                            Logger.i(it.message)
                            callback(it.message)
                            it.body?.let {
                                _fileId.postValue(it.first())
                            }

                        }, onFail = {
                            Logger.e(it)
                            callback("图片上传失败")
                        }, typeToken = object : TypeToken<ApiResponse<List<Int>>>() {}.type),
                        fileList = arrayOf(cacheFile)
                    )
                }
            }

        }
    }


    private fun clean() {
        name.clean()
        desc.clean()
        _picture.postValue(null)
    }

    /**
     *
     *
     * @param callback
     */
    fun register(callback: (value: String) -> Unit) {
        val fileId = _fileId.value
        if (fileId != null &&
            name.statusForm.value == FormStatus.Valid && desc.statusForm.value == FormStatus.Valid
        ) {
            viewModelScope.launch {
                HttpClient.post(
                    Api.buildUrl(AssociationApi.Register),
                    HttpCallback<Boolean>("注册社团", onSuccess = {
                        Logger.i(it.message)
                        callback(it.message)
                        if (it.body == true) {
                            clean()
                        }
                    }, onFail = {
                        Logger.e(it)
                    }, typeToken = object : TypeToken<ApiResponse<Boolean>>() {}.type),
                    jsonParam = AssociationRegVo(
                        name = name.getValue(),
                        desc = desc.getValue(),
                        fileId = fileId,
                        associationId = _checkInfo.value?.body?.associationVo?.associationId,
                        token = TokenManager.getToken()
                    )
                )
            }
        } else {
            callback(UNKNOW_ERROR)
        }
    }

}