package com.gyf.csams.association.ui

import android.content.Intent
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.material.*
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import com.gyf.csams.R
import com.gyf.csams.association.model.*
import com.gyf.csams.module.QUESTION_MIN_SIZE
import com.gyf.csams.uikit.Background
import com.gyf.csams.uikit.BackgroundImage
import com.gyf.lib.uikit.BaseTextField
import com.gyf.lib.uikit.Body
import com.gyf.lib.uikit.MainColumnFrame
import com.gyf.lib.uikit.ScaffoldModel
import com.gyf.lib.util.BottomButton
import com.orhanobut.logger.Logger


/**
 * 题库管理
 *
 */
class ExamActivity : ComponentActivity() {

    private val activityType: ExamActivityType
        get() {
            return intent?.getSerializableExtra(ExamActivityType::name.name) as ExamActivityType
        }

    private val associationId: Int
        get() {
            val id = intent.getIntExtra(
                AssociationActivity::class.java.name,
                0
            )
            return if (id == 0) throw IllegalArgumentException("不存在社团id，获取失败") else id
        }

    private val joinId: Int
        get() {
            val id = intent.getIntExtra(
                AuditJoinActivity::class.java.name,
                0
            )
            return if (id == 0) throw IllegalArgumentException("不存在申请记录id，获取失败") else id
        }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContent {
            Body {
                MainColumnFrame(background = {
                    Background(
                        image = BackgroundImage.Exam,
                        alpha = 0.6F
                    )
                }) {
                    Title()
                    Spacer(modifier = Modifier.height(10.dp))
                    Exam(modifier = Modifier.weight(0.8F))
                }
            }
        }
    }

    @Composable
    private fun ErrorRadioButton(onClick: () -> Unit = {}) {
        RadioButton(
            selected = true,
            colors =
            RadioButtonDefaults.colors(disabledColor = MaterialTheme.colors.error),
            onClick = onClick,
            enabled = false
        )
    }

    @Composable
    private fun RightRadioButton(selected: Boolean, onClick: () -> Unit = {}) {
        RadioButton(
            selected = selected,
            colors =
            RadioButtonDefaults.colors(disabledColor = MaterialTheme.colors.primary),
            onClick = onClick,
            enabled = false
        )
    }

    /**
     * 标题
     *
     * @param modifier
     */
    @Composable
    private fun Title(modifier: Modifier = Modifier, model: ExamViewModel = viewModel()) {
        val data by model.data.observeAsState()
        Column(modifier = modifier, horizontalAlignment = Alignment.CenterHorizontally) {
            Text(text = activityType.menuName, style = MaterialTheme.typography.h5)
            data?.size?.let {
                Text(buildString {
                    append("共有${if (activityType == ExamActivityType.SET_EXAM) it - 1 else it}道题目")
                    if (it < QUESTION_MIN_SIZE) {
                        append(",还差${QUESTION_MIN_SIZE - it}道题目才可以生成入团笔试题")
                    }
                }, style = MaterialTheme.typography.h6.copy(color = MaterialTheme.colors.primary))
            }
            when (activityType) {
                ExamActivityType.Answer -> Row(
                    modifier = Modifier.fillMaxWidth(),
                    horizontalArrangement = Arrangement.SpaceEvenly
                ) {
                    Row {
                        RightRadioButton(selected = true)
                        Text(text = "表示正确选项")
                    }
                    Row {
                        ErrorRadioButton()
                        Text(text = "表示错误选项")
                    }
                }
                ExamActivityType.SET_EXAM -> Row(modifier = Modifier.fillMaxWidth()) {
                    Text(text = "通过单选按钮设置正确答案")
                }
                ExamActivityType.JOIN_Association -> Row(modifier = Modifier.fillMaxWidth()) {
                    Text(text = "通过单选按钮选择正确答案")
                }
            }

        }
    }


    @Composable
    private fun ExamChild(it: Exam, examHeight: Dp, isAdd: Boolean = false) {
        val questionWeight = 0.3F
        when (it) {
            is OpenQuestionsVo -> ExamOQ(
                openQuestionsVo = it,
                modifier = Modifier.height(examHeight * questionWeight),
                isAdd = isAdd,
            )
            is ChoiceQuestionVo -> ExamCQ(
                choiceQuestionVo = it,
                modifier = Modifier.height(examHeight),
                questionWeight = questionWeight,
                isAdd = isAdd
            )
        }
    }

    /**
     * 题目列表
     *
     * @param modifier
     * @param model
     */
    @Composable
    private fun Exam(
        modifier: Modifier = Modifier,
        model: ExamViewModel = viewModel(),
        scaffoldModel: ScaffoldModel = viewModel(),
        examHeight: Dp = 350.dp
    ) {
        LaunchedEffect(associationId) {
            when (activityType) {
                ExamActivityType.JOIN_Association, ExamActivityType.SET_EXAM -> model.load(
                    associationId = associationId,
                    activityType = activityType
                )
                ExamActivityType.Answer -> model.showAnswer(joinId = joinId) {
                    scaffoldModel.update(message = "答卷加载成功", actionLabel = "关闭提示")
                }
            }

        }
        val listState = rememberLazyListState()
        val data by model.data.observeAsState()
        val newExam by model.newExam.observeAsState()
        Column(modifier = modifier) {
            LazyColumn(state = listState, modifier = Modifier.weight(0.8F)) {
                data?.forEach {
                    item {
                        ExamChild(it = it, examHeight = examHeight)
                        Spacer(modifier = Modifier.height(20.dp))
                    }
                }
                newExam?.let {
                    item {
                        Column {
//                        OutlinedButton(onClick = { model.switchType(it) }) {
//                            Text(text = "切换到${if (newExam is ChoiceQuestionVo) "开放题" else "选择题"}")
//                        }
                            ExamChild(it = it, examHeight = examHeight, isAdd = true)
                        }
                    }
                }

            }

            Divider(color = MaterialTheme.colors.background)
            Spacer(modifier = Modifier.height(30.dp))
            when (activityType) {
                ExamActivityType.SET_EXAM -> {
                    BottomButton(
                        modifier = Modifier.fillMaxWidth(),
                        confirmDesc = R.string.update_exam,
                        enabled = data?.size ?: 0 > 0
                    ) {
                        model.updateExam(associationId = associationId) {
                            scaffoldModel.update(
                                message = it, actionLabel = "关闭提示"
                            )
                        }
                    }
                }
                ExamActivityType.JOIN_Association -> {
                    val allowPostAnswer by model.allowPostAnswer.observeAsState(false)
                    BottomButton(
                        modifier = Modifier.fillMaxWidth(),
                        confirmDesc = R.string.post_answer,
                        backDesc = R.string.cancel_apply,
                        enabled = allowPostAnswer
                    ) {
                        model.postAnswer(associationId = associationId) {
                            scaffoldModel.update(message = it)
                            setResult(RESULT_OK, Intent().apply {
                                putExtra(ExamActivity::class.java.name, it)
                            })
                            finish()
                        }
                    }
                }
                ExamActivityType.Answer -> {
                    Row(
                        modifier = Modifier.fillMaxWidth(),
                        horizontalArrangement = Arrangement.Center
                    ) {
                        OutlinedButton(onClick = { onBackPressed() }) {
                            Text(text = "返回")
                        }
                    }

                }
            }
        }
    }

    /**
     * 问题
     *
     * @param modifier
     * @param exam
     */
    @Composable
    private fun Question(
        modifier: Modifier = Modifier,
        exam: Exam
    ) {
        exam._question.let {
            BaseTextField(
                form = it,
                modifier = modifier
                    .fillMaxSize()
                    .background(color = MaterialTheme.colors.background),
                readOnly = activityType != ExamActivityType.SET_EXAM,
                showTrailingIcon = activityType == ExamActivityType.SET_EXAM
            )
        }

    }

    /**
     * 操作按钮
     *
     */
    @Composable
    private fun ActionButton(
        modifier: Modifier = Modifier, isAdd: Boolean,
        model: ExamViewModel = viewModel(),
        scaffoldModel: ScaffoldModel = viewModel(),
        exam: Exam
    ) {
        Box(
            contentAlignment = Alignment.Center,
            modifier = modifier
        ) {
            IconButton(onClick = {
                if (isAdd) {
                    if (exam.check()) {
                        scaffoldModel.update(
                            message = model.addTip,
                            actionLabel = model.actionLabel
                        ) {
                            model.addQuestion()
                        }
                    } else {
                        scaffoldModel.update(message = model.questionIsNull, actionLabel = "知道了")
                    }

                } else {
                    scaffoldModel.update(
                        message = model.deleteTip,
                        actionLabel = model.actionLabel
                    ) {
                        model.deleteQuestion(exam = exam)
                    }
                }
            }) {
                Icon(
                    painter = painterResource(id = if (isAdd) R.drawable.ic_add_select else R.drawable.ic_sami_select),
                    contentDescription = null
                )
            }
        }

    }


    /**
     * 开放题
     *
     * @param openQuestionsVo
     */
    @Composable
    private fun ExamOQ(
        modifier: Modifier = Modifier,
        openQuestionsVo: OpenQuestionsVo,
        isAdd: Boolean = false,
        model: ExamViewModel = viewModel()
    ) {
        val data by model.data.observeAsState()
        Row(modifier = modifier) {
            Logger.i("index=${data?.indexOf(openQuestionsVo)}")
            data?.indexOf(openQuestionsVo)?.let {
                Text(text = "${it}.", modifier = Modifier.weight(0.05F))
            }
            Question(
                exam = openQuestionsVo, modifier =
                if (activityType == ExamActivityType.SET_EXAM)
                    Modifier
                        .weight(0.8F)
                        .fillMaxHeight()
                else
                    Modifier
            )
            if (activityType == ExamActivityType.SET_EXAM) {
                ActionButton(
                    modifier = Modifier
                        .weight(0.15F)
                        .fillMaxHeight(),
                    isAdd = isAdd,
                    exam = openQuestionsVo
                )
            }
        }
    }

    /**
     * 选择题
     *
     * @param choiceQuestionVo
     */
    @Composable
    private fun ExamCQ(
        modifier: Modifier = Modifier,
        choiceQuestionVo: ChoiceQuestionVo,
        isAdd: Boolean = false,
        model: ExamViewModel = viewModel(),
        questionWeight: Float
    ) {
        Row(modifier = modifier) {
            model.data.value?.let {

                Text(
                    text = "${
                        if (it.indexOf(choiceQuestionVo) == -1) it.size else it.indexOf(
                            choiceQuestionVo
                        ) + 1
                    }."
                )
            }
            Column(
                modifier = if (activityType == ExamActivityType.SET_EXAM)
                    Modifier
                        .weight(0.8F)
                        .fillMaxHeight()
                else
                    Modifier
            ) {
                Question(
                    exam = choiceQuestionVo,
                    modifier = Modifier
                        .fillMaxWidth()
                        .weight(questionWeight)
                )
                Card(
                    backgroundColor = MaterialTheme.colors.secondaryVariant,
                    modifier = Modifier
                        .fillMaxWidth()
                        .weight(1 - questionWeight)
                ) {

                    choiceQuestionVo._answers.apply {
                        Column {
                            forEach {
                                Row(
                                    modifier = Modifier
                                        .fillMaxWidth()
                                        .weight(1F / ANSWER_SIZE)
                                ) {
                                    val answerIndex: Int = indexOf(it)


                                    when (activityType) {
                                        ExamActivityType.SET_EXAM -> {
                                            RadioButton(
                                                selected = choiceQuestionVo.rightAnswer == answerIndex,
                                                onClick = {
                                                    model.update(
                                                        oldExam = choiceQuestionVo,
                                                        newExam = choiceQuestionVo.copy(rightAnswer = answerIndex)
                                                    )
                                                })
                                        }
                                        ExamActivityType.JOIN_Association -> {
                                            RadioButton(
                                                selected = choiceQuestionVo.chooseOption == answerIndex,
                                                onClick = {
                                                    model.update(
                                                        oldExam = choiceQuestionVo,
                                                        newExam = choiceQuestionVo.copy(chooseOption = answerIndex)
                                                    )
                                                    model.checkChooseAnswer()
                                                })
                                        }
                                        ExamActivityType.Answer -> {
                                            when {
                                                choiceQuestionVo.rightAnswer == answerIndex -> RightRadioButton(
                                                    selected = true
                                                )
                                                choiceQuestionVo.chooseOption != choiceQuestionVo.rightAnswer && choiceQuestionVo.chooseOption == answerIndex ->
                                                    ErrorRadioButton()
                                                else -> RadioButton(
                                                    selected = false,
                                                    onClick = {},
                                                    enabled = false
                                                )
                                            }

                                        }
                                    }

                                    BaseTextField(
                                        form = it,
                                        enabled = activityType != ExamActivityType.Answer,
                                        showTrailingIcon = activityType == ExamActivityType.SET_EXAM
                                    )

                                }
                            }
                        }
                    }
                }
            }

            ActionButton(
                modifier = Modifier
                    .weight(0.2F)
                    .fillMaxHeight(),
                isAdd = isAdd,
                exam = choiceQuestionVo
            )

        }
    }
}
