package com.gyf.lib.util

import com.google.gson.*
import com.gyf.csams.module.ApiResponse
import com.gyf.csams.module.Token
import com.orhanobut.logger.Logger
import okhttp3.*
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.RequestBody.Companion.asRequestBody
import okhttp3.RequestBody.Companion.toRequestBody
import java.io.File
import java.io.IOException
import java.lang.reflect.Field
import java.lang.reflect.Type
import java.net.SocketTimeoutException


object SuperclassExclusionStrategy : ExclusionStrategy {
    //序列化反序列化白名单
    val whiteClass = listOf<Class<*>>(Token::class.java)

    override fun shouldSkipClass(arg0: Class<*>?): Boolean {
        return false
    }

    override fun shouldSkipField(fieldAttributes: FieldAttributes): Boolean {
        val fieldName = fieldAttributes.name
        val theClass = fieldAttributes.declaringClass
        return isFieldInSuperclass(theClass, fieldName)
    }

    private fun isFieldInSuperclass(subclass: Class<*>, fieldName: String): Boolean {
        var superclass = subclass.superclass
        var field: Field?
        while (superclass != null && !whiteClass.contains(subclass)) {
            field = getField(superclass, fieldName)
            if (field != null) return true
            superclass = superclass.superclass
        }
        return false
    }

    private fun getField(theClass: Class<*>, fieldName: String): Field? {
        return try {
            theClass.getDeclaredField(fieldName)
        } catch (e: Exception) {
            null
        }
    }
}

val HTTP_CALLBACK_GSON_CONFIG: Gson = GsonBuilder()
    .addSerializationExclusionStrategy(SuperclassExclusionStrategy)
    .addDeserializationExclusionStrategy(SuperclassExclusionStrategy)
    .create()

object HttpClient {
    private val httpClient: OkHttpClient = OkHttpClient()

    private val JSON_CONTENT_TYPE = "application/json; charset=UTF-8".toMediaType()


    /**
     * 构建url查询参数
     *
     * @param params
     * @return
     */
    private fun buildQueryParams(params: Map<String, String>?): String {
        return if (params?.isNotEmpty() == true) {
            val urlPath = StringBuilder("?")
            for (i in params) {
                urlPath.append(i.key).append("=").append(i.value)
            }

            urlPath.toString()
        } else {
            ""
        }
    }

    /**
     * 构建表单参数
     *
     * @param params
     * @return
     */
    private fun buildFormBody(params: Map<String, String>?): FormBody {
        val builder = FormBody.Builder()
        if (params?.isNotEmpty() == true) {
            for (item in params) {
                builder.add(item.key, item.value)
            }
        }
        return builder.build()
    }

    /**
     * HTTP GET
     *
     * @param url
     * @param callback
     * @param params
     */
    fun get(url: String, callback: Callback, params: Map<String, String>? = null) {
        Logger.i("request url=$url")
        val request = Request.Builder()
            .url(url.plus(buildQueryParams(params = params)))
            .build()
        val call = httpClient.newCall(request)
        call.enqueue(callback)
    }

    /**
     * 发送json表单
     *
     * @param url
     * @param callback
     * @param jsonParam TODO 改写成reified
     */
    fun post(url: String, callback: Callback, jsonParam: Any) {
        Logger.i("request url=$url")
        val jsonBody = HTTP_CALLBACK_GSON_CONFIG.toJson(jsonParam)
        Logger.json(jsonBody)
        val request = Request.Builder()
            .url(url)
            .post(body = jsonBody.toRequestBody(contentType = JSON_CONTENT_TYPE))
            .build()
        val call = httpClient.newCall(request)
        call.enqueue(callback)
    }

    fun <T> postAsync(url: String, jsonParam: Any, type: Type): ApiResponse<T>? {
        Logger.i("request url=$url")
        val jsonBody = HTTP_CALLBACK_GSON_CONFIG.toJson(jsonParam)
        Logger.json(jsonBody)
        val request = Request.Builder()
            .url(url)
            .post(body = jsonBody.toRequestBody(contentType = JSON_CONTENT_TYPE))
            .build()

        return httpClient.newCall(request).execute().use { response ->
            return@use response.body?.string()?.run<String, ApiResponse<T>?> {
                Logger.i("jsonStr:${this}")
                Gson().fromJson(this, type)
            }
        }
    }

    /**
     * 上传文件
     *
     * @param url
     * @param callback
     */
    fun uploadFile(
        url: String,
        callback: Callback,
        params: Map<String, String>? = null,
        vararg fileList: File
    ) {
        Logger.i("request url=$url")
        val token = TokenManager.getToken()
        val body = MultipartBody.Builder()
            .addFormDataPart("id", token.id.toString())
            .addFormDataPart("token", token.token)
            .addFormDataPart("createTime", token.createTime.toString())
        params?.forEach {
            body.addFormDataPart(it.key, it.value)
        }
        fileList.withIndex().forEach {
            body.addFormDataPart(
                "file${it.index}",
                it.value.name,
                it.value.asRequestBody()
            )
        }
        val request = Request.Builder()
            .url(url)
            .post(body = body.build())
            .build()
        val call = httpClient.newCall(request)
        call.enqueue(callback)
    }
}


interface GsonBuilderInterface {
    val gson: Gson
}

open class HttpCallback<T>(
    private val action: String,
    private val onSuccess: (res: ApiResponse<T>) -> Unit = { Logger.i(it.message) },
    private val onFail: (error: String) -> Unit = { Logger.e(it) },
    private val typeToken: Type
) : Callback, GsonBuilderInterface {
    override val gson: Gson = HTTP_CALLBACK_GSON_CONFIG

    override fun onFailure(call: Call, e: IOException) {
        when (e) {
            is SocketTimeoutException -> {
                onFail("${action}失败，网络连接超时！")
            }
            else -> {
                onFail("${action}失败，请联系管理员")
            }
        }
        Logger.e(e, "${action}请求失败，发生IO异常")
    }

    override fun onResponse(call: Call, response: Response) {
        if (response.code == 200) {
            val body = response.body
            if (body != null && body.contentType()?.subtype == "json") {
                val jsonRes = body.string()
                Logger.i(jsonRes)
                try {
                    val res: ApiResponse<T> = gson.fromJson(jsonRes, typeToken)
                    onSuccess(res)
                } catch (e: JsonSyntaxException) {
                    Logger.e(e, "json反序列化成${typeToken}失败")
                    onFail("json反序列化成${typeToken}失败")
                } catch (e: Exception) {
                    Logger.e(e, "发生未知错误")
                    onFail("发生未知错误")
                }

            } else {
                onFail("${action}失败，请联系管理员")
                Logger.e("无法解析${action}请求响应数据：,响应码:${response.code},${response.body}")
            }
        } else {
            onFail("${action}失败，请联系管理员")
            Logger.e("${action}失败，请求响应码:${response.code}")
        }
    }
}

