package com.gyf.lib.util

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.room.*
import com.gyf.csams.module.*
import com.orhanobut.logger.Logger


/**
 * 登陆令牌
 */
@Entity
class ClientToken(
    @PrimaryKey
    override val id: Int,
    @ColumnInfo
    override val token: String,
    @ColumnInfo
    override val createTime: Long,
) : Token(token, createTime, id)


data class OnlyToken(
    override val clientType: ClientType = TokenManager.getClientType()
) : ClientBaseVo() {
    override val token: ClientToken = TokenManager.getToken()
}

@Dao
interface TokenDao {
    @Query("select * from ClientToken")
    suspend fun queryAll(): List<ClientToken>

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun save(clientToken: ClientToken)

    @Delete
    suspend fun delete(user: ClientToken)

    @Query("delete from ClientToken")
    suspend fun deleteAll()
}



object TokenManager {
    private val ownInfo = MutableLiveData<OwnInfoVo?>()
    private val personInfo = MutableLiveData<PersonInfoVo?>()
    private val clientToken = MutableLiveData<ClientToken?>()
    fun update(ownInfo: OwnInfoVo) {
        this.ownInfo.postValue(ownInfo)
        ownInfo.token.let {
            clientToken.postValue(
                ClientToken(
                    id = it.id,
                    token = it.token,
                    createTime = it.createTime
                )
            )
        }
        when (ownInfo) {
            is ManagerVo -> this.personInfo.postValue(
                ManagerInfoVo(
                    duty = ownInfo.duty,
                    name = ownInfo.name,
                    headImg = ownInfo.headImg,
                    desc = ownInfo.desc
                )
            )
            is UserVo -> this.personInfo.postValue(
                UserInfoVo(
                    name = ownInfo.name,
                    headImg = ownInfo.headImg,
                    desc = ownInfo.desc
                )
            )
            else -> throw IllegalArgumentException("token初始化失败")
        }
        Logger.i("token刷新完成")
    }

    fun clear() {
        ownInfo.postValue(null)
        personInfo.postValue(null)
        clientToken.postValue(null)
    }

    fun getClientType(): ClientType {
        return when (ownInfo.value) {
            is UserVo -> ClientType.Foreground
            is ManagerVo -> ClientType.Background
            else -> throw IllegalArgumentException("个人信息还没有初始化")
        }
    }

    fun getUserInfo(): UserVo? {
        return ownInfo.value as? UserVo
    }

    fun getPersonInfoData(): LiveData<PersonInfoVo?> {
        return personInfo
    }

    fun getManagerInfo(): ManagerVo? {
        return ownInfo.value as? ManagerVo
    }

    fun getToken(): ClientToken {
        return clientToken.value ?: throw IllegalArgumentException("token没有初始化，非法调用")
    }
}


@Database(entities = [ClientToken::class], version = 1, exportSchema = false)
abstract class AppDatabase : RoomDatabase() {
    abstract fun tokenDao(): TokenDao
    private val mIsDatabaseCreated = MutableLiveData<Boolean>()

    /**
     * Check whether the database already exists and expose it via [.getDatabaseCreated]
     */
    private fun updateDatabaseCreated(context: Context) {
        if (context.getDatabasePath(DATABASE_NAME).exists()) {
            setDatabaseCreated()
        }
    }

    private fun setDatabaseCreated() {
        mIsDatabaseCreated.postValue(true)
    }

    val databaseCreated: LiveData<Boolean>
        get() = mIsDatabaseCreated

    companion object {
        private var sInstance: AppDatabase? = null
        const val DATABASE_NAME = "basic-sample-db"
        fun getInstance(context: Context): AppDatabase? {
            if (sInstance == null) {
                synchronized(AppDatabase::class.java) {
                    if (sInstance == null) {
                        sInstance =
                            buildDatabase(context.applicationContext)
                        sInstance!!.updateDatabaseCreated(context.applicationContext)
                    }
                }
            }
            return sInstance
        }


        private fun buildDatabase(appContext: Context): AppDatabase {
            return Room.databaseBuilder(appContext, AppDatabase::class.java, DATABASE_NAME)
                .build()
        }


    }
}